# !/usr/bin/env python3
#
#  Optimize blocksize of apps/mmm_block.cpp
#
#  This is an extremely simplified version meant only for tutorials
#
from __future__ import print_function
# import adddeps  #  fix sys.path
import sys

import opentuner
from opentuner import ConfigurationManipulator
from opentuner import IntegerParameter
from opentuner import EnumParameter
from opentuner import MeasurementInterface
from opentuner import Result


class GccFlagsTuner(MeasurementInterface):

  def manipulator(self):
    """
    Define the search space by creating a
    ConfigurationManipulator
    """
    manipulator = ConfigurationManipulator()


    """
    ======================
    APPLICATION PROPERTIES
    ======================
    """
    # spark.driver.maxResultSize
    manipulator.add_parameter(EnumParameter('spark_driver_maxresultsize', ['500m', '1g', '4g']))
    # spark.driver.memory
    manipulator.add_parameter(EnumParameter('spark_driver_memory', ['500m', '1g', '8g']))
    # spark.executor.memory
    manipulator.add_parameter(EnumParameter('spark_executor_memory', ['10g', '50g', '80g']))
    # spark.executor.pyspark.memory
    manipulator.add_parameter(EnumParameter('spark_executor_pyspark_memory', ['4g', '8g', '16g']))
    # spark.executor.memoryOverhead
    manipulator.add_parameter(EnumParameter('spark_executor_memory_overhead', ['500m', '2g', '4g']))
    # spark.executor.instances
    manipulator.add_parameter(EnumParameter('spark_executor_instances', ['1', '4', '10', '20']))

    # NOT USED
    # spark.master
    # spark.app.name
    # spark.submit.deployMode
    # spark.driver.cores # only in cluster mode
    # spark.driver.memoryOverhead # only in cluster mode
    # spark.resources.discoveryPlugin
    # spark.driver.resource.{resourceName}.amount
    # spark.driver.resource.{resourceName}.discoveryScript
    # spark.driver.resource.{resourceName}.vendor
    # spark.executor.resource.{resourceName}.vendor
    # spark.executor.resource.{resourceName}.discoveryScript
    # spark.executor.resource.{resourceName}.amount
    # spark.extraListeners
    # spark.local.dir # might be important for SSD vs HDD
    # spark.log.callerContext
    # spark.driver.supervise # Only in standalone or Mesos cluster mode
    # spark.logConf
    # spark.driver.log.dfsDir
    # spark.driver.log.persistToDfs.enabled
    # spark.history.fs.driverlog.cleaner.enabled
    # spark.history.fs.driverlog.cleaner.maxAge
    # spark.driver.log.persistToDfs.enabled
    # spark.driver.log.dfsDir
    # spark.history.fs.driverlog.cleaner.enabled
    # spark.driver.log.layout
    # spark.driver.log.dfsDir
    # spark.driver.log.allowErasureCoding

    """
    ===================
    RUNTIME ENVIRONMENT
    ===================
    """
    # spark.python.worker.memory
    manipulator.add_parameter(EnumParameter('spark_python_worker_memory', ['64m', '512m', '4g']))
    # spark.python.worker.reuse
    manipulator.add_parameter(EnumParameter('spark_python_worker_reuse', ['true', 'false']))

    # NOT USED
    # spark.driver.extraClassPath
    # spark.driver.defaultJavaOptions
    # spark.driver.extraJavaOptions
    # spark.driver.extraLibraryPath
    # spark.driver.userClassPathFirst
    # spark.executor.extraClassPath
    # spark.executor.defaultJavaOptions
    # spark.executor.extraJavaOptions
    # spark.executor.extraLibraryPath
    # spark.executor.logs.rolling.enableCompression
    # spark.executor.logs.rolling.maxSize
    # spark.executor.logs.rolling.strategy
    # spark.executor.logs.rolling.time.interval
    # spark.executor.logs.rolling.maxRetainedFiles
    # spark.executor.userClassPathFirst
    # spark.executorEnv.[EnvironmentVariableName]
    # spark.redaction.regex
    # spark.python.profile
    # spark.python.profile.dump
    # spark.files
    # spark.submit.pyFiles
    # spark.jars
    # spark.jars.packages
    # spark.jars.excludes
    # spark.jars.ivy
    # spark.jars.ivySettings
    # spark.jars.repositories
    # spark.pyspark.driver.python
    # spark.pyspark.python

    """
    ================
    SHUFFLE BEHAVIOR
    ================
    """
    # spark.reducer.maxSizeInFlight
    manipulator.add_parameter(EnumParameter('spark_reducer_maxsizeinflight', ['12m', '48m', '100m']))
    # spark.reducer.maxReqsInFlight
    manipulator.add_parameter(EnumParameter('spark_reducer_maxreqsinflight', ['2147483647', '200000000']))
    # spark.reducer.maxBlocksInFlightPerAddress
    manipulator.add_parameter(EnumParameter('spark_reducer_maxblocksinflightperaddress', ['2147483647', '200000000']))
    # spark.shuffle.compress
    manipulator.add_parameter(EnumParameter('spark_shuffle_compress', ['true', 'false']))
    # spark.io.compression.codec
    manipulator.add_parameter(EnumParameter('spark_io_compression_codec', ['snappy', 'lzf', 'lz4']))
    # spark.shuffle.file.buffer
    manipulator.add_parameter(EnumParameter('spark_shuffle_file_buffer', ['8k', '32k', '128k']))
    # spark.shuffle.io.maxRetries
    manipulator.add_parameter(EnumParameter('spark_shuffle_io_maxretries', ['2', '3', '6']))
    # spark.shuffle.io.numConnectionsPerPeer
    manipulator.add_parameter(EnumParameter('spark_shuffle_io_numconnectionsperpeer', ['1', '3']))
    # spark.shuffle.io.preferDirectBufs
    manipulator.add_parameter(EnumParameter('spark_shuffle_io_preferdirectbufs', ['true', 'false']))
    # spark.shuffle.io.retryWait
    manipulator.add_parameter(EnumParameter('spark_shuffle_io_retrywait', ['3s', '5s', '8s']))
    # spark.shuffle.service.enabled
    manipulator.add_parameter(EnumParameter('spark_shuffle_service_enabled', ['true', 'false']))
    # spark.shuffle.service.index.cache.size
    manipulator.add_parameter(EnumParameter('spark_shuffle_service_index_cache_size', ['10m','100m', '500m']))
    # spark.shuffle.sort.bypassMergeThreshold
    manipulator.add_parameter(EnumParameter('spark_shuffle_sort_bypassmergethreshold', ['50', '200', '800']))
    # spark.shuffle.spill.compress
    manipulator.add_parameter(EnumParameter('spark_shuffle_spill_compress', ['true', 'false']))

    # NOT USED
    # spark.shuffle.maxChunksBeingTransferred
    # spark.shuffle.accurateBlockThreshold
    # spark.shuffle.registration.timeout
    # spark.shuffle.registration.maxAttempts
    # spark.shuffle.service.port
    # spark.shuffle.io.backLog

    """
    ========
    SPARK UI
    ========
    """
    # NOT USED
    # spark.eventLog.logBlockUpdates.enabled
    # spark.eventLog.enabled
    # spark.eventLog.longForm.enabled
    # spark.eventLog.compress
    # spark.eventLog.enabled
    # spark.eventLog.compression.codec
    # spark.eventLog.erasureCoding.enabled
    # spark.eventLog.dir
    # spark.eventLog.enabled
    # spark.eventLog.enabled
    # spark.eventLog.overwrite
    # spark.eventLog.buffer.kb
    # spark.eventLog.rolling.enabled
    # spark.eventLog.rolling.maxFileSize
    # spark.eventLog.rolling.enabled=true
    # spark.ui.dagGraph.retainedRootRDDs
    # spark.ui.enabled
    # spark.ui.killEnabled
    # spark.ui.liveUpdate.period
    # spark.ui.liveUpdate.minFlushPeriod
    # spark.ui.port
    # spark.ui.retainedJobs
    # spark.ui.retainedStages
    # spark.ui.retainedTasks
    # spark.ui.reverseProxy
    # spark.ui.reverseProxyUrl
    # spark.ui.proxyRedirectUri
    # spark.ui.showConsoleProgress
    # spark.ui.custom.executor.log.url
    # spark.worker.ui.retainedExecutors
    # spark.worker.ui.retainedDrivers
    # spark.sql.ui.retainedExecutions
    # spark.streaming.ui.retainedBatches
    # spark.ui.retainedDeadExecutors
    # spark.ui.filters
    # spark.&lt;class name of filter&gt;.param.&lt;param name&gt;=&lt;value&gt;
    # spark.ui.filters=com.test.filter1
    # spark.com.test.filter1.param.name1=foo
    # spark.com.test.filter1.param.name2=bar
    # spark.ui.requestHeaderSize

    """
    =============================
    COMPRESSION AND SERIALIZATION
    =============================
    """
    # spark.broadcast.compress
    manipulator.add_parameter(EnumParameter('spark_broadcast_compress', ['true', 'false']))
    # spark.checkpoint.compress
    manipulator.add_parameter(EnumParameter('spark_checkpoint_compress', ['true', 'false']))
    # spark.io.compression.lz4.blockSize
    manipulator.add_parameter(EnumParameter('spark_io_compression_lz4_blocksize', ['16k', '32k', '64k']))
    # spark.io.compression.snappy.blockSize
    manipulator.add_parameter(EnumParameter('spark_io_compress_snappy_blocksize', ['16k', '32k', '64k']))
    # spark.io.compression.zstd.level
    manipulator.add_parameter(EnumParameter('spark_io_compression_zstd_level', ['0', '1', '4']))
    # spark.io.compression.zstd.bufferSize
    manipulator.add_parameter(EnumParameter('spark_io_compression_zstd_buffersize', ['8k', '32k', '128k']))
    # spark.kryo.referenceTracking
    manipulator.add_parameter(EnumParameter('spark_kryo_referencetracking', ['true', 'false']))
    # spark.kryo.registrationRequired
    manipulator.add_parameter(EnumParameter('spark_kryo_registrationrequired', ['true', 'false']))
    # spark.kryo.unsafe
    manipulator.add_parameter(EnumParameter('spark_kryo_unsafe', ['true', 'false']))
    # spark.kryoserializer.buffer.max
    manipulator.add_parameter(EnumParameter('spark_kryoserializer_buffer_max', ['32m', '64m', '128m']))
    # spark.kryoserializer.buffer
    manipulator.add_parameter(EnumParameter('spark_kryoserializer_buffer', ['32k', '64k', '128k']))
    # spark.rdd.compress
    manipulator.add_parameter(EnumParameter('spark_rdd_compress', ['true', 'false']))
    # spark.serializer
    manipulator.add_parameter(EnumParameter('spark_serializer', ['org.apache.spark.serializer.JavaSerializer', 'org.apache.spark.serializer.KryoSerializer']))
    # spark.serializer.objectStreamReset
    manipulator.add_parameter(EnumParameter('spark_serializer_objectstreamreset', ['-1', '50', '100', '200']))
    
    # NOT USED
    # spark.kryo.classesToRegister
    # spark.kryo.registrator

    """
    =================
    MEMORY MANAGEMENT
    =================
    """
    # spark.memory.fraction
    manipulator.add_parameter(EnumParameter('spark_memory_fraction', ['0.4', '0.6', '0.7']))
    # spark.memory.storageFraction
    manipulator.add_parameter(EnumParameter('spark_memory_storagefraction', ['0.2', '0.5', '0.8']))
    # spark.memory.offHeap.enabled
    manipulator.add_parameter(EnumParameter('spark_memory_offheap_enabled', ['true', 'false']))
    # spark.memory.offHeap.size
    manipulator.add_parameter(EnumParameter('spark_memory_offheap_size', ['25m', '50m']))
    # spark.storage.replication.proactive
    manipulator.add_parameter(EnumParameter('spark_storage_repllication_proactive', ['true', 'false']))
    # spark.cleaner.periodicGC.interval
    manipulator.add_parameter(EnumParameter('spark_cleaner_periodicgc_interval', ['15min', '30min', '60min']))
    # spark.cleaner.referenceTracking
    manipulator.add_parameter(EnumParameter('spark_cleaner_referencetracking', ['true', 'false']))
    # spark.cleaner.referenceTracking.blocking
    manipulator.add_parameter(EnumParameter('spark_spark_cleaner_referencetracking_blocking', ['true', 'false']))
    # spark.cleaner.referenceTracking.blocking.shuffle
    manipulator.add_parameter(EnumParameter('spark_cleaner_referencetracking_blocking_shuffle', ['true', 'false']))
    # spark.cleaner.referenceTracking.cleanCheckpoints
    manipulator.add_parameter(EnumParameter('spark_cleaner_referencetracking_cleancheckpoints', ['true', 'false']))

    """
    ==================
    EXECUTION BEHAVIOR
    ==================
    """
    # spark.broadcast.blockSize
    manipulator.add_parameter(EnumParameter('spark_broadcast_blocksize', ['1m', '4m', '16m']))
    # spark.broadcast.checksum
    manipulator.add_parameter(EnumParameter('spark_broadcast_checksum', ['true', 'false']))
    # spark.executor.cores
    manipulator.add_parameter(EnumParameter('spark_executor_cores', ['1', '4', '8', '16', '24', '48']))
    # spark.default.parallelism
    manipulator.add_parameter(EnumParameter('spark_default_parallelism', ['1', '8', '24']))
    # spark.executor.heartbeatInterval
    manipulator.add_parameter(EnumParameter('spark_executor_heartbeatinterval', ['5s', '10s', '20s']))
    # spark.files.fetchTimeout
    manipulator.add_parameter(EnumParameter('spark_files_fetchtimeout', ['30s', '60s', '120s']))
    # spark.files.useFetchCache
    manipulator.add_parameter(EnumParameter('spark_files_usefetchcache', ['true', 'false']))
    # spark.files.maxPartitionBytes
    manipulator.add_parameter(EnumParameter('spark_files_maxpartitionbytes', ['70000000', '134217728', '190000000']))
    # spark.files.openCostInBytes
    manipulator.add_parameter(EnumParameter('spark_files_opencostinbytes', ['2000000', '4194304', '9000000']))
    # spark.storage.memoryMapThreshold
    manipulator.add_parameter(EnumParameter('spark_storage_memorymapthreshold', ['1m', '2m', '4m']))

    # NOT USED
    # spark.hadoop.mapreduce.fileoutputcommitter.algorithm.version
    # spark.files.overwrite
    # spark.hadoop.cloneConf
    # spark.hadoop.validateOutputSpecs

    """
    ================
    EXECUTOR METRICS
    ================
    """
    # spark.executor.metrics.pollingInterval
    manipulator.add_parameter(EnumParameter('spark_executor_metrics_pollinginterval', ['0', '2s', '5s']))

    # NOT USED
    # spark.eventLog.logStageExecutorMetrics
    # spark.executor.processTreeMetrics.enabled

    """
    ==========
    NETWORKING
    ==========
    """
    # spark.rpc.message.maxSize
    manipulator.add_parameter(EnumParameter('spark_rpc_message_maxsize', ['16', '128', '512'])) # mallon xwris "m"
    # spark.network.timeout
    manipulator.add_parameter(EnumParameter('spark_network_timeout', ['60s', '120s', '240s']))
    # spark.storage.blockManagerSlaveTimeoutMs
    # spark.shuffle.io.connectionTimeout
    # spark.rpc.lookupTimeout
    # spark.network.io.preferDirectBufs
    # spark.port.maxRetries
    manipulator.add_parameter(EnumParameter('spark_port_maxretries', ['8', '16', '32'] ))
    # spark.rpc.numRetries
    manipulator.add_parameter(EnumParameter('spark_rpc_numretries', ['2', '3', '6']))
    # spark.rpc.retry.wait
    manipulator.add_parameter(EnumParameter('spark_rpc_retry_wait', ['2s', '3s', '6s',]))
    # spark.rpc.askTimeout
    manipulator.add_parameter(EnumParameter('spark_rpc_asktimeout', ['60s', '120s', '240s']))
    # spark.network.timeout
    # spark.rpc.lookupTimeout
    manipulator.add_parameter(EnumParameter('spark_rpc_lookuptimeout', ['60s', '120s', '240s']))
    # spark.core.connection.ack.wait.timeout
    manipulator.add_parameter(EnumParameter('spark_core_connection_ack_wait_timeout', ['60s', '120s', '240s']))
    # spark.network.maxRemoteBlockSizeFetchToMem
    manipulator.add_parameter(EnumParameter('spark_network_maxremoteblocksizefetchtomem', ['2147483147', '200000000']))
    # spark.blockManager.port

    # NOT USED
    # spark.driver.blockManager.port
    # spark.driver.bindAddress
    # spark.driver.host
    # spark.driver.port
    # spark.rpc.io.backLog
    # spark.network.timeout

    """
    ==========
    SCHEDULING
    ==========
    """
    # spark.locality.wait
    manipulator.add_parameter(EnumParameter('spark_locality_wait', ['1s', '3s', '10s']))
    # spark.locality.wait.node
    manipulator.add_parameter(EnumParameter('spark_locality_wait_node', ['1s', '3s', '10s']))
    # spark.locality.wait.process
    manipulator.add_parameter(EnumParameter('spark_locality_wait_process', ['2s', '3s', '6s']))
    # spark.locality.wait.rack
    manipulator.add_parameter(EnumParameter('spark_locality_wait_rack', ['2s', '3s', '6s']))
    # spark.scheduler.maxRegisteredResourcesWaitingTime
    manipulator.add_parameter(EnumParameter('spark_scheduler_maxregisteredresourceswaitingtime', ['15s', '30s', '60s']))
    # spark.scheduler.minRegisteredResourcesRatio
    manipulator.add_parameter(EnumParameter('spark_scheduler_minregisteredresourcesratio', ['15s', '30s', '60s']))
    # spark.scheduler.mode
    manipulator.add_parameter(EnumParameter('spark_scheduler_mode', ['FIFO', 'FAIR']))
    # spark.scheduler.revive.interval
    manipulator.add_parameter(EnumParameter('spark_scheduler_revive_interval', ['1s', '3s']))
    # spark.scheduler.listenerbus.eventqueue.capacity
    manipulator.add_parameter(EnumParameter('spark_scheduler_listenerbus_eventqueue_capacity', ['1000', '10000', '50000']))
    # spark.scheduler.listenerbus.eventqueue.shared.capacity
    manipulator.add_parameter(EnumParameter('spark_scheduler_listenerbus_eventqueue_shared_capacity', ['1000', '10000', '50000']))
    # spark.scheduler.listenerbus.eventqueue.appStatus.capacity
    manipulator.add_parameter(EnumParameter('spark_scheduler_listenerbus_eventqueue_appstatus_capacity', ['1000', '10000', '50000']))
    # spark.scheduler.listenerbus.eventqueue.executorManagement.capacity
    manipulator.add_parameter(EnumParameter('spark_scheduler_listenerbus_eventqueue_executormanagement_capacity', ['1000', '10000', '50000']))
    # spark.scheduler.listenerbus.eventqueue.eventLog.capacity
    manipulator.add_parameter(EnumParameter('spark_scheduler_listenerbus_eventqueue_eventlog_capacity', ['1000', '10000', '50000']))
    # spark.scheduler.listenerbus.eventqueue.streams.capacity
    manipulator.add_parameter(EnumParameter('spark_scheduler_listenerbus_eventqueue_streams_capacity', ['1000', '10000', '50000']))
    # spark.scheduler.blacklist.unschedulableTaskSetTimeout
    manipulator.add_parameter(EnumParameter('spark_scheduler_blacklist_unschedulabletasksettimeout', ['60s', '120s', '240s']))
    # spark.blacklist.enabled
    manipulator.add_parameter(EnumParameter('spark_blacklist_enabled', ['true', 'false']))
    # spark.blacklist.timeout
    manipulator.add_parameter(EnumParameter('spark_blacklist_timeout', ['30min', '60min', '120min']))
    # spark.blacklist.task.maxTaskAttemptsPerExecutor
    manipulator.add_parameter(IntegerParameter('spark_blacklist_task_maxattemptsperexecutor', 1, 3))
    # spark.blacklist.task.maxTaskAttemptsPerNode
    manipulator.add_parameter(IntegerParameter('spark_blacklist_task_maxtaskattemptspernode', 2, 4))
    # spark.blacklist.stage.maxFailedTasksPerExecutor
    manipulator.add_parameter(IntegerParameter('spark_blacklist_stage_maxfailedtasksperexecutor', 2, 4))
    # spark.blacklist.stage.maxFailedExecutorsPerNode
    manipulator.add_parameter(IntegerParameter('spark_blacklist_stage_maxfailedexecutorspernode', 2, 4))
    # spark.blacklist.application.maxFailedTasksPerExecutor
    manipulator.add_parameter(IntegerParameter('spark_blacklist_application_maxfailedtasksperexecutor', 2, 4))
    # spark.blacklist.application.maxFailedExecutorsPerNode
    manipulator.add_parameter(IntegerParameter('spark_blacklist_application_maxfailedexecutorspernode', 2, 4))
    # spark.blacklist.killBlacklistedExecutors
    manipulator.add_parameter(EnumParameter('spark_blacklist_killblacklistedexecutors', ['true', 'false']))
    # spark.speculation
    manipulator.add_parameter(EnumParameter('spark_speculation', ['true', 'false']))
    # spark.speculation.interval
    manipulator.add_parameter(EnumParameter('spark_speculation_interval', ['10ms', '100ms', '500ms']))
    # spark.speculation.multiplier
    manipulator.add_parameter(EnumParameter('spark_speculation_multiplier', ['1.1', '1.5', '5']))
    # spark.speculation.quantile
    manipulator.add_parameter(EnumParameter('spark_speculation_quantile', ['0.5','0.75', '0.85']))
    # spark.task.cpus
    manipulator.add_parameter(IntegerParameter('spark_task_cpus', 1, 3))
    # spark.task.maxFailures
    manipulator.add_parameter(EnumParameter('spark_task_maxfailures', ['2', '4', '8']))
    # spark.task.reaper.enabled
    manipulator.add_parameter(EnumParameter('spark_task_reaper_enabled', ['true', 'false']))
    # spark.task.reaper.pollingInterval
    manipulator.add_parameter(EnumParameter('spark_task_reaper_pollinginterval', ['5s', '10s', '20s']))
    # spark.task.reaper.threadDump
    manipulator.add_parameter(EnumParameter('spark_task_reaper_threaddump', ['true', 'false']))
    # spark.stage.maxConsecutiveAttempts
    manipulator.add_parameter(EnumParameter('spark_stage_maxconsecutiveattempts', ['2', '4', '8']))

    # NOT USED
    # spark.speculation.task.duration.threshold
    # spark.cores.max
    # spark.blacklist.application.fetchFailure.enabled
    # spark.task.resource.{resourceName}.amount
    # spark.executor.resource.{resourceName}.amount
    # spark.task.reaper.killTimeout

    """
    ======================
    BARRIER EXECUTION MODE
    ======================
    """
    # spark.barrier.sync.timeout
    # spark.scheduler.barrier.maxConcurrentTasksCheck.interval
    # spark.scheduler.barrier.maxConcurrentTasksCheck.maxFailures
    """
    ==================
    DYNAMIC ALLOCATION
    ==================
    """
    # spark.dynamicAllocation.enabled
    manipulator.add_parameter(EnumParameter('spark_dynamicallocation_enabled', ['true', 'false']))
    # spark.dynamicAllocation.executorIdleTimeout
    manipulator.add_parameter(EnumParameter('spark_dynamicallocation_executoridletimeout', ['10s', '60s', '240s']))
    # spark.dynamicAllocation.initialExecutors
    manipulator.add_parameter(EnumParameter('spark_dynamicallocation_initialexecutors', ['0', '1', '4']))
    # spark.dynamicAllocation.minExecutors
    manipulator.add_parameter(EnumParameter('spark_dynamicallocation_minexecutors', ['0', '4', '10']))
    # spark.dynamicAllocation.executorAllocationRatio
    manipulator.add_parameter(EnumParameter('spark_dynamicallocation_executorallocationratio', ['0.1', '0.5', '1']))
    # spark.dynamicAllocation.shuffleTracking.enabled
    manipulator.add_parameter(EnumParameter('spark_dynamicallocation_shuffletracking_enabled', ['true', 'false']))

    # NOT USED
    # spark.dynamicAllocation.cachedExecutorIdleTimeout
    # spark.dynamicAllocation.maxExecutors
    # spark.dynamicAllocation.schedulerBacklogTimeout
    # spark.dynamicAllocation.sustainedSchedulerBacklogTimeout
    # spark.dynamicAllocation.shuffleTracking.timeout

    """
    =====================
    THREAD CONFIGURATIONS
    =====================
    """
    # spark.{driver|executor}.rpc.netty.dispatcher.numThreads
    # spark.{driver|executor}.rpc.io.clientThreads
    # spark.{driver|executor}.rpc.netty.dispatcher.numThreads
    """
    =========
    SPARK SQL
    =========
    """
    # spark.sql.adaptive.advisoryPartitionSizeInBytes
    # spark.sql.adaptive.shuffle.targetPostShuffleInputSize
    # spark.sql.adaptive.coalescePartitions.enabled
    # spark.sql.adaptive.coalescePartitions.initialPartitionNum
    # spark.sql.adaptive.coalescePartitions.minPartitionNum
    # spark.sql.adaptive.enabled
    # spark.sql.adaptive.localShuffleReader.enabled
    # spark.sql.adaptive.skewJoin.enabled
    # spark.sql.adaptive.skewJoin.skewedPartitionFactor
    # spark.sql.adaptive.skewJoin.skewedPartitionThresholdInBytes
    # spark.sql.ansi.enabled
    # spark.sql.autoBroadcastJoinThreshold
    # spark.sql.avro.compression.codec
    # spark.sql.avro.deflate.level
    # spark.sql.broadcastTimeout
    # spark.sql.catalog.spark_catalog
    # spark.sql.cbo.enabled
    # spark.sql.cbo.joinReorder.dp.star.filter
    # spark.sql.cbo.joinReorder.dp.threshold
    # spark.sql.cbo.joinReorder.enabled
    # spark.sql.cbo.planStats.enabled
    # spark.sql.cbo.starSchemaDetection
    # spark.sql.columnNameOfCorruptRecord
    # spark.sql.csv.filterPushdown.enabled
    # spark.sql.datetime.java8API.enabled
    # spark.sql.debug.maxToStringFields
    # spark.sql.defaultCatalog
    # spark.sql.execution.arrow.enabled
    # spark.sql.execution.arrow.fallback.enabled
    # spark.sql.execution.arrow.maxRecordsPerBatch
    # spark.sql.execution.arrow.pyspark.enabled
    # spark.sql.execution.arrow.enabled
    # spark.sql.execution.arrow.pyspark.fallback.enabled
    # spark.sql.execution.arrow.fallback.enabled
    # spark.sql.execution.arrow.sparkr.enabled
    # spark.sql.execution.pandas.udf.buffer.size
    # spark.buffer.size
    # spark.buffer.size
    # spark.sql.files.ignoreCorruptFiles
    # spark.sql.files.ignoreMissingFiles
    # spark.sql.files.maxPartitionBytes
    # spark.sql.files.maxRecordsPerFile
    # spark.sql.function.concatBinaryAsString
    # spark.sql.function.eltOutputAsString
    # spark.sql.groupByAliases
    # spark.sql.groupByOrdinal
    # spark.sql.hive.convertInsertingPartitionedTable
    # spark.sql.hive.convertMetastoreParquet
    # spark.sql.hive.convertMetastoreCtas
    # spark.sql.hive.convertMetastoreParquet
    # spark.sql.hive.convertMetastoreOrc
    # spark.sql.hive.convertMetastoreParquet
    # spark.sql.hive.convertMetastoreParquet.mergeSchema
    # spark.sql.hive.filesourcePartitionFileCacheSize
    # spark.sql.hive.manageFilesourcePartitions
    # spark.sql.hive.metastorePartitionPruning
    # spark.sql.hive.thriftServer.async
    # spark.sql.hive.verifyPartitionPath
    # spark.sql.hive.version
    # spark.sql.hive.metastore.version
    # spark.sql.inMemoryColumnarStorage.batchSize
    # spark.sql.inMemoryColumnarStorage.compressed
    # spark.sql.inMemoryColumnarStorage.enableVectorizedReader
    # spark.sql.jsonGenerator.ignoreNullFields
    # spark.sql.legacy.allowHashOnMapType
    # spark.sql.mapKeyDedupPolicy
    # spark.sql.maven.additionalRemoteRepositories
    # spark.sql.maxPlanStringLength
    # spark.sql.optimizer.dynamicPartitionPruning.enabled
    # spark.sql.optimizer.excludedRules
    # spark.sql.orc.columnarReaderBatchSize
    # spark.sql.orc.compression.codec
    # spark.sql.orc.compression.codec
    # spark.sql.orc.enableVectorizedReader
    # spark.sql.orc.filterPushdown
    # spark.sql.orc.mergeSchema
    # spark.sql.orderByOrdinal
    # spark.sql.parquet.binaryAsString
    # spark.sql.parquet.columnarReaderBatchSize
    # spark.sql.parquet.compression.codec
    # spark.sql.parquet.compression.codec
    # spark.sql.parquet.enableVectorizedReader
    # spark.sql.parquet.filterPushdown
    # spark.sql.parquet.int96AsTimestamp
    # spark.sql.parquet.int96TimestampConversion
    # spark.sql.parquet.mergeSchema
    # spark.sql.parquet.outputTimestampType
    # spark.sql.parquet.recordLevelFilter.enabled
    # spark.sql.parquet.respectSummaryFiles
    # spark.sql.parquet.writeLegacyFormat
    # spark.sql.parser.quotedRegexColumnNames
    # spark.sql.pivotMaxValues
    # spark.sql.pyspark.jvmStacktrace.enabled
    # spark.sql.redaction.options.regex
    # spark.sql.redaction.string.regex
    # spark.redaction.string.regex
    # spark.redaction.string.regex
    # spark.sql.repl.eagerEval.enabled
    # spark.sql.repl.eagerEval.maxNumRows
    # spark.sql.repl.eagerEval.truncate
    # spark.sql.session.timeZone
    # spark.sql.shuffle.partitions
    # spark.sql.sources.bucketing.enabled
    # spark.sql.sources.bucketing.maxBuckets
    # spark.sql.sources.default
    # spark.sql.sources.parallelPartitionDiscovery.threshold
    # spark.sql.sources.partitionColumnTypeInference.enabled
    # spark.sql.sources.partitionOverwriteMode
    # spark.sql.statistics.fallBackToHdfs
    # spark.sql.statistics.histogram.enabled
    # spark.sql.statistics.size.autoUpdate.enabled
    # spark.sql.storeAssignmentPolicy
    # spark.sql.streaming.checkpointLocation
    # spark.sql.streaming.continuous.epochBacklogQueueSize
    # spark.sql.streaming.disabledV2Writers
    # spark.sql.streaming.fileSource.cleaner.numThreads
    # spark.sql.streaming.forceDeleteTempCheckpointLocation
    # spark.sql.streaming.metricsEnabled
    # spark.sql.streaming.multipleWatermarkPolicy
    # spark.sql.streaming.noDataMicroBatches.enabled
    # spark.sql.streaming.numRecentProgressUpdates
    # spark.sql.streaming.stopActiveRunOnRestart
    # spark.sql.streaming.stopTimeout
    # spark.sql.thriftserver.scheduler.pool
    # spark.sql.thriftserver.ui.retainedSessions
    # spark.sql.thriftserver.ui.retainedStatements
    # spark.sql.variable.substitute
    """
    ========================
    STATIC SQL CONFIGURATION
    ========================
    """
    # spark.sql.event.truncate.length
    # spark.sql.extensions
    # spark.sql.hive.metastore.barrierPrefixes
    # spark.sql.hive.metastore.jars
    # spark.sql.hive.metastore.version
    # spark.sql.hive.metastore.sharedPrefixes
    # spark.sql.hive.metastore.version
    # spark.sql.hive.thriftServer.singleSession
    # spark.sql.legacy.sessionInitWithConfigDefaults
    # spark.sql.queryExecutionListeners
    # spark.sql.streaming.streamingQueryListeners
    # spark.sql.streaming.ui.enabled
    # spark.sql.streaming.ui.retainedProgressUpdates
    # spark.sql.streaming.ui.retainedQueries
    # spark.sql.ui.retainedExecutions
    # spark.sql.warehouse.dir
    """
    ===============
    SPARK STREAMING
    ===============
    """
    # spark.streaming.backpressure.enabled
    # spark.streaming.receiver.maxRate
    # spark.streaming.backpressure.initialRate
    # spark.streaming.blockInterval
    # spark.streaming.receiver.maxRate
    # spark.streaming.receiver.writeAheadLog.enable
    # spark.streaming.unpersist
    # spark.streaming.stopGracefullyOnShutdown
    # spark.streaming.kafka.maxRatePerPartition
    # spark.streaming.kafka.minRatePerPartition
    # spark.streaming.ui.retainedBatches
    # spark.streaming.driver.writeAheadLog.closeFileAfterWrite
    # spark.streaming.receiver.writeAheadLog.closeFileAfterWrite
    """
    ======
    SPARKR
    ======
    """
    # spark.r.numRBackendThreads
    # spark.r.command
    # spark.r.driver.command
    # spark.r.shell.command
    # spark.r.shell.command
    # spark.r.backendConnectionTimeout
    # spark.r.heartBeatInterval
    """
    ======
    GRAPHX
    ======
    """
    # spark.graphx.pregel.checkpointInterval
    """
    ======
    DEPLOY
    ======
    """
    # spark.deploy.recoveryMode
    # spark.deploy.zookeeper.url
    # spark.deploy.zookeeper.dir

    return manipulator

  def run(self, desired_result, input, limit):
    """
    Compile and run a given configuration then
    return performance
    """
    cfg = desired_result.configuration.data

    # run_cmd = 'rm out'
    # run_result = self.call_program(run_cmd)
    # assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.shuffle.file.buffer		"'
    run_cmd += str(cfg['spark_shuffle_file_buffer'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.shuffle.sort.bypassMergeThreshold		"'
    run_cmd += str(cfg['spark_shuffle_sort_bypassmergethreshold'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.spark_speculation.interval		"'
    run_cmd += str(cfg['spark_speculation_interval'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.spark_speculation.multiplier		"'
    run_cmd += str(cfg['spark_speculation_multiplier'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.spark_speculation.quantile		"'
    run_cmd += str(cfg['spark_speculation_quantile'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.broadcast.blockSize		"'
    run_cmd += str(cfg['spark_broadcast_blocksize'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.io.compression.codec		"'
    run_cmd += str(cfg['spark_io_compression_codec'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.io.compression.lz4.blockSize		"'
    run_cmd += str(cfg['spark_io_compression_lz4_blocksize'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.io.compression.snappy.blockSize		"'
    run_cmd += str(cfg['spark_io_compress_snappy_blocksize'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    """
    run_cmd = 'echo "spark.kryo.referenceTracking		"'
    run_cmd += str(cfg['kryo_ref_track'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0
    """

    run_cmd = 'echo "spark.kryoserializer.buffer.max		"'
    run_cmd += str(cfg['spark_kryoserializer_buffer_max'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.kryoserializer.buffer		"'
    run_cmd += str(cfg['spark_kryoserializer_buffer'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    """
    run_cmd = 'echo "spark.driver.cores		"'
    run_cmd += str(cfg['driver_cores'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0
    """

    run_cmd = 'echo "spark.executor.cores		"'
    run_cmd += str(cfg['spark_executor_cores'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    
    run_cmd = 'echo "spark.driver.memory		"'
    run_cmd += str(cfg['spark_driver_memory'])

    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 
	
    run_cmd = 'echo "spark.storage.memoryMapThreshold		"'
    run_cmd += str(cfg['spark_storage_memorymapthreshold'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.network.timeout		"'
    run_cmd += str(cfg['spark_network_timeout'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.locality.wait		"'
    run_cmd += str(cfg['spark_locality_wait'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.scheduler.revive.interval		"'
    run_cmd += str(cfg['spark_scheduler_revive_interval'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.task.maxFailures		"'
    run_cmd += str(cfg['spark_task_maxfailures'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0 

    run_cmd = 'echo "spark.shuffle.compress		"'
    run_cmd += str(cfg['spark_shuffle_compress'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.memory.fraction		"'
    run_cmd += str(cfg['spark_memory_fraction'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.shuffle.spill.compress		"'
    run_cmd += str(cfg['spark_shuffle_spill_compress'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.spark_speculation		"'
    run_cmd += str(cfg['spark_speculation'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.broadcast.compress		"'
    run_cmd += str(cfg['spark_broadcast_compress'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.rdd.compress		"'
    run_cmd += str(cfg['spark_rdd_compress'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.memory.storageFraction		"'
    run_cmd += str(cfg['spark_memory_storagefraction'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    """
    if cfg['default_parall']!='0':
        run_cmd = 'echo "spark.default.parallelism		"'
        run_cmd += str(cfg['default_parall'])
        run_cmd += ' >> out'
        run_result = self.call_program(run_cmd)
        assert run_result['returncode'] == 0
    """	

    run_cmd = 'echo "spark.memory.offHeap.enabled		"'
    run_cmd += str(cfg['spark_memory_offheap_enabled'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.memory.offHeap.size		"'
    run_cmd += str(cfg['spark_memory_offheap_size'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

# ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
    run_cmd = 'echo "spark.driver.maxResultSize		"'
    run_cmd += str(cfg['spark_driver_maxresultsize'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.reducer.maxReqsInFlight		"'
    run_cmd += str(cfg['spark_reducer_maxreqsinflight'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.reducer.maxBlocksInFlightPerAddress		"'
    run_cmd += str(cfg['spark_reducer_maxblocksinflightperaddress'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.maxRemoteBlockSizeFetchToMem		"'
    run_cmd += str(cfg['spark_network_maxremoteblocksizefetchtomem'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.shuffle.io.maxRetries		"'
    run_cmd += str(cfg['spark_shuffle_io_maxretries'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.shuffle.io.numConnectionsPerPeer		"'
    run_cmd += str(cfg['spark_shuffle_io_numconnectionsperpeer'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.shuffle.io.preferDirectBufs		"'
    run_cmd += str(cfg['spark_shuffle_io_preferdirectbufs'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.shuffle.io.retryWait		"'
    run_cmd += str(cfg['spark_shuffle_io_retrywait'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.kryo.unsafe		"'
    run_cmd += str(cfg['spark_kryo_unsafe'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.serializer.objectStreamReset		"'
    run_cmd += str(cfg['spark_serializer_objectstreamreset'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.storage.replication.proactive		"'
    run_cmd += str(cfg['spark_storage_repllication_proactive'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.cleaner.periodicGC.interval		"'
    run_cmd += str(cfg['spark_cleaner_periodicgc_interval'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.cleaner.referenceTracking		"'
    run_cmd += str(cfg['spark_cleaner_referencetracking'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.cleaner.referenceTracking.blocking		"'
    run_cmd += str(cfg['spark_spark_cleaner_referencetracking_blocking'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.cleaner.referenceTracking.blocking.shuffle		"'
    run_cmd += str(cfg['spark_cleaner_referencetracking_blocking_shuffle'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.cleaner.referenceTracking.cleanCheckpoints		"'
    run_cmd += str(cfg['spark_cleaner_referencetracking_cleancheckpoints'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.executor.heartbeatInterval		"'
    run_cmd += str(cfg['spark_executor_heartbeatinterval'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.files.fetchTimeout		"'
    run_cmd += str(cfg['spark_files_fetchtimeout'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.files.maxPartitionBytes		"'
    run_cmd += str(cfg['spark_files_maxpartitionbytes'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.files.openCostInBytes		"'
    run_cmd += str(cfg['spark_files_opencostinbytes'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.rpc.message.maxSize		"'
    run_cmd += str(cfg['spark_rpc_message_maxsize'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.port.maxRetries		"'
    run_cmd += str(cfg['spark_port_maxretries'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.rpc.numRetries		"'
    run_cmd += str(cfg['spark_rpc_numretries'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.rpc.retry.wait		"'
    run_cmd += str(cfg['spark_rpc_retry_wait'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.rpc.askTimeout		"'
    run_cmd += str(cfg['spark_rpc_asktimeout'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.rpc.lookupTimeout		"'
    run_cmd += str(cfg['spark_rpc_lookuptimeout'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.core.connection.ack.wait.timeout		"'
    run_cmd += str(cfg['spark_core_connection_ack_wait_timeout'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.locality.wait.node		"'
    run_cmd += str(cfg['spark_locality_wait_node'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.locality.wait.process		"'
    run_cmd += str(cfg['spark_locality_wait_process'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.locality.wait.rack		"'
    run_cmd += str(cfg['spark_locality_wait_rack'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.scheduler.maxRegisteredResourcesWaitingTime		"'
    run_cmd += str(cfg['spark_scheduler_maxregisteredresourceswaitingtime'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.scheduler.mode		"'
    run_cmd += str(cfg['spark_scheduler_mode'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.scheduler.listenerbus.eventqueue.capacity		"'
    run_cmd += str(cfg['spark_scheduler_listenerbus_eventqueue_capacity'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.scheduler.blacklist.unschedulableTaskSetTimeout		"'
    run_cmd += str(cfg['spark_scheduler_blacklist_unschedulabletasksettimeout'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.blacklist.enabled		"'
    run_cmd += str(cfg['spark_blacklist_enabled'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.blacklist.timeout		"'
    run_cmd += str(cfg['spark_blacklist_timeout'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.blacklist.task.maxTaskAttemptsPerExecutor		"'
    run_cmd += str(cfg['spark_blacklist_task_maxattemptsperexecutor'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.blacklist.task.maxTaskAttemptsPerNode		"'
    run_cmd += str(cfg['spark_blacklist_task_maxtaskattemptspernode'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.blacklist.stage.maxFailedTasksPerExecutor		"'
    run_cmd += str(cfg['spark_blacklist_stage_maxfailedtasksperexecutor'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.blacklist.stage.maxFailedExecutorsPerNode		"'
    run_cmd += str(cfg['spark_blacklist_stage_maxfailedexecutorspernode'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.blacklist.application.maxFailedTasksPerExecutor		"'
    run_cmd += str(cfg['spark_blacklist_application_maxfailedtasksperexecutor'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.blacklist.application.maxFailedExecutorsPerNode		"'
    run_cmd += str(cfg['spark_blacklist_application_maxfailedexecutorspernode'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.blacklist.killBlacklistedExecutors		"'
    run_cmd += str(cfg['spark_blacklist_killblacklistedexecutors'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.task.cpus		"'
    cpu = cfg['spark_task_cpus']
    if int(cfg['spark_task_cpus']) > int(cfg['spark_executor_cores']):
        cpu = cfg['spark_executor_cores']
    run_cmd += str(cpu)
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.task.reaper.enabled		"'
    run_cmd += str(cfg['spark_task_reaper_enabled'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.task.reaper.pollingInterval		"'
    run_cmd += str(cfg['spark_task_reaper_pollinginterval'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.task.reaper.threadDump		"'
    run_cmd += str(cfg['spark_task_reaper_threaddump'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.stage.maxConsecutiveAttempts		"'
    run_cmd += str(cfg['spark_stage_maxconsecutiveattempts'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    """
    NEW METRICS
    """
    run_cmd = 'echo "spark.executor.pyspark.memory		"'
    run_cmd += str(cfg['spark_executor_pyspark_memory'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.reducer.maxSizeInFlight    "'
    run_cmd += str(cfg['spark_reducer_maxsizeinflight'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.shuffle.service.enabled    "'
    run_cmd += str(cfg['spark_shuffle_service_enabled'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.shuffle.service.index.cache.size    "'
    run_cmd += str(cfg['spark_shuffle_service_index_cache_size'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.checkpoint.compress    "'
    run_cmd += str(cfg['spark_checkpoint_compress'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.io.compression.zstd.level    "'
    run_cmd += str(cfg['spark_io_compression_zstd_level'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.io.compression.zstd.buffersize    "'
    run_cmd += str(cfg['spark_io_compression_zstd_buffersize'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.kryo.referenceTracking    "'
    run_cmd += str(cfg['spark_kryo_referencetracking'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.kryo.registrationrequired    "'
    run_cmd += str(cfg['spark_kryo_registrationrequired'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.serializer    "'
    run_cmd += str(cfg['spark_serializer'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.broadcast.checksum    "'
    run_cmd += str(cfg['spark_broadcast_checksum'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.default.parallelism    "'
    run_cmd += str(cfg['spark_default_parallelism'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.files.useFetchCache    "'
    run_cmd += str(cfg['spark_files_usefetchcache'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.executor.metrics.pollingInterval   "'
    run_cmd += str(cfg['spark_executor_metrics_pollinginterval'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.python.worker.memory   "'
    run_cmd += str(cfg['spark_python_worker_memory'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.python.worker.reuse   "'
    run_cmd += str(cfg['spark_python_worker_reuse'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.memory.storageFraction   "'
    run_cmd += str(cfg['spark_memory_storagefraction'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.scheduler.minRegisteredResourcesRatio    "'
    run_cmd += str(cfg['spark_scheduler_minregisteredresourcesratio'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0
    

    run_cmd = 'echo "spark.scheduler.listenerbus.eventqueue.shared.capacity	    "'
    run_cmd += str(cfg['spark_scheduler_listenerbus_eventqueue_shared_capacity'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.scheduler.listenerbus.eventqueue.appStatus.capacity    "'
    run_cmd += str(cfg['spark_scheduler_listenerbus_eventqueue_appstatus_capacity'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.scheduler.listenerbus.eventqueue.executorManagement.capacity	    "'
    run_cmd += str(cfg['spark_scheduler_listenerbus_eventqueue_executormanagement_capacity'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.scheduler.listenerbus.eventqueue.streams.capacity	    "'
    run_cmd += str(cfg['spark_scheduler_listenerbus_eventqueue_streams_capacity'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.scheduler.listenerbus.eventqueue.eventLog.capacity		    "'
    run_cmd += str(cfg['spark_scheduler_listenerbus_eventqueue_eventlog_capacity'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.dynamicAllocation.enabled    "'
    run_cmd += str(cfg['spark_dynamicallocation_enabled'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.executor.instances    "'
    run_cmd += str(cfg['spark_executor_instances'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.dynamicAllocation.enabled    "'
    run_cmd += str(cfg['spark_dynamicallocation_enabled'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.dynamicAllocation.executorIdleTimeout   "'
    run_cmd += str(cfg['spark_dynamicallocation_executoridletimeout'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.dynamicAllocation.initialExecutors   "'
    run_cmd += str(cfg['spark_dynamicallocation_initialexecutors'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.dynamicAllocation.minExecutors   "'
    run_cmd += str(cfg['spark_dynamicallocation_minexecutors'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.dynamicAllocation.executorAllocationRatio   "'
    run_cmd += str(cfg['spark_dynamicallocation_executorallocationratio'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.dynamicAllocation.shuffleTracking.enabled   "'
    run_cmd += str(cfg['spark_dynamicallocation_shuffletracking_enabled'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0

    run_cmd = 'echo "spark.executor.memory		"'
    # exec_mem = float(cfg['spark_executor_memory'])/4		# pairnoun floor oi prakseis    
    # exec_mem = exec_mem - 0.1 * exec_mem    # -> 10% memory overhead
    # exec_mem = exec_mem * 1000
    # exec_mem = int(exec_mem)
    run_cmd += str(cfg['spark_executor_memory'])
    run_cmd += ' >> out'
    run_result = self.call_program(run_cmd)
    assert run_result['returncode'] == 0
	
	# in light node, 4cores per machine, 4gb memory per machine -> need to be taken into account

    # return Result(time=run_result['time'])
    exit(0)

  def save_final_config(self, configuration):
    """called at the end of tuning"""
    print("Optimal block size written to mmm_final_config.json:", configuration.data)
    self.manipulator().save_to_file(configuration.data,
                                    'mmm_final_config.json')


if __name__ == '__main__':
  argparser = opentuner.default_argparser()
  GccFlagsTuner.main(argparser.parse_args())
